#include <IRremote.h>
#include <Servo.h>
/*
Button 1 : Light Bulb ON
Button 4 : Light Bulb OFF

Button 2 : Open Window 
Button 5 : Close Window

Button 3 : bring up the curtain
Button 6 : bring down the curtain*/

const long ESP8266_BAUDRATE = 115200L;
const unsigned long DEFAULT_TIMEOUT = 5000L;
String _dataRecived;

int irPin     = 5;
int pinData   = 5;
int TMP = A0;
int LIGHT = A2;
int INFRAROUGE = 7;
int AMPOULE = 6;
int RECIVER = 5;

int GAZ = A1;
int BUZZER = 8;
int UP = 2, DOWN = 3;
Servo Servo;
bool windowOpened = false;

int MONTE = 11, DESCENTE = 10, CONFIG = 9;

///// Etat des composants dans le Remote
int remoteLamp = 0;
int remoteServo = 0;
int remoteMotor= 0;

int nextTimeout =0;

//Librairie capteur infrarouge
IRrecv irrecv(RECIVER);
decode_results results;


void setup()
{
  Serial.begin(ESP8266_BAUDRATE);
  while (!Serial) {
    ; // wait for serial port to connect. Needed for native USB port only
  }
  pinMode(INFRAROUGE, INPUT);
  pinMode(AMPOULE, OUTPUT);
  
  pinMode(GAZ, INPUT);
  pinMode(BUZZER, OUTPUT);
  Servo.attach(4);
  Servo.write(0);
  
  pinMode(MONTE, OUTPUT);
  pinMode(DESCENTE, OUTPUT);
  pinMode(CONFIG, OUTPUT);
  
  pinMode(UP, INPUT);
  pinMode(DOWN, INPUT);
  
  analogWrite(CONFIG , map(3,0,5,0,255) );
  /*********** INITIALIZE COMMUNICATION *********/
  sendCommandAT("AT+RST", DEFAULT_TIMEOUT, false, false, true);
  sendCommandAT("AT", DEFAULT_TIMEOUT, true, false, true);
  sendCommandAT("AT+GMR", DEFAULT_TIMEOUT, true, false, true);
  //sendCommandAT("AT+CWMODE=1", DEFAULT_TIMEOUT, true, false, true);
  sendCommandAT("AT+CWLAP", DEFAULT_TIMEOUT, true, false, true);
  // Connectez-vous au Wifi Simulator a l'aide de la commande AT + CWJAP
  sendCommandAT("AT+CWJAP=\"Simulator Wifi\",\"\"", DEFAULT_TIMEOUT, true, false, true);
  // Affiche l'ESP8266 IP
  sendCommandAT("AT+CIFSR", DEFAULT_TIMEOUT, true, false, true);
  //sendCommandAT("AT+CWMODE=3", DEFAULT_TIMEOUT, true, false, true);
  
  // Ouvre le canal de communication TCP avec le site Web a l'aide de la commande AT + CIPSTART
  sendCommandAT("AT+CIPSTART=\"TCP\",\"api-smartkitchen.000webhostapp.com\",80", DEFAULT_TIMEOUT, true, false, true);
  
  nextTimeout = millis();
  
  irrecv.enableIRIn(); 
}

void loop()
{
  if (irrecv.decode(&results)) {
    //Returns the value retrieved from the infrared sensor
    unsigned int value = results.value; 
    irrecv.resume(); // Reprendre la réception
  }
  detecterMouvement();
  RemoteOnOff();
}
void RemoteOnOff(){
  if (irrecv.decode(&results)) {
    
    //Returns the value retrieved from the infrared sensor
    unsigned int value = results.value;
    Serial.println(value);
    switch(value){
      case 34935: // Num 2 // Open Window
      	Servo.write(90);
      	break;
      case 43095: // Num 5 // Close Window
      	Servo.write(0);
      	break;
      case 18615: // Num 3 // Open Curtain
      	digitalWrite(MONTE,HIGH);
  		digitalWrite(DESCENTE,LOW);
      	break;
      case 26775: // Num 6 // lose Curtain
      	digitalWrite(DESCENTE,HIGH);
  		digitalWrite(MONTE,LOW);
      	break;
      case 2295: //num 1 Light Bulb ON
      	digitalWrite(AMPOULE, HIGH);
      	break;
      case 10455: //num 4 Light Bulb OFF
      	digitalWrite(AMPOULE, LOW);
      	break;
    }
    irrecv.resume(); // Resume reception
  }
  if(digitalRead(UP)) {
    digitalWrite(MONTE,LOW);windowOpened = true;};
  if(digitalRead(DOWN)) {
    digitalWrite(DESCENTE,LOW); windowOpened = false;};
  
  if(analogRead(GAZ)>120) Alert(true);
  else Alert(false);
  
  if(millis()> nextTimeout){
  	// Request site data using HTTP GET
    
    String Req = "GET /index.php?getData=true&gaz=";
        Req.concat((analogRead(GAZ)>120)? 1 : 0);
        Req.concat("&motion=");
        Req.concat(digitalRead(INFRAROUGE));
        Req.concat("&temp=");
        Req.concat(getTemperature());
        Req.concat(" HTTP/1.1\r\nHost: {URL:example.com}\r\n\r\n");
    String preReq = "AT+CIPSEND=";
    	preReq.concat(Req.length());
    //Pre Req ou il ya la taille de la requette
    sendCommandAT(preReq , DEFAULT_TIMEOUT, false, true, false);
    // Envoie de la requette
    sendCommandAT(Req , 60000, false, false, true);
  	nextTimeout = millis() + 500;
  }
  
}
void detecterMouvement(){
  if(digitalRead(INFRAROUGE))
    digitalWrite(AMPOULE , HIGH);
  if(digitalRead(INFRAROUGE))
    digitalWrite(AMPOULE, LOW);
}

int getTemperature(){
  int reading = analogRead(TMP);
  float voltage = reading * 5.0;
  voltage /= 1024.0; 
  float temperatureC = (voltage - 0.5) * 100 ;
     return (int)temperatureC;

}
void Alert(bool a){
  if(a){
  	digitalWrite(BUZZER,HIGH);
    Servo.write(90);
    if(!windowOpened){
    	digitalWrite(DESCENTE,LOW);
  		digitalWrite(MONTE,HIGH);
      windowOpened=true;
    }
  }else{
  	digitalWrite(BUZZER,LOW);}}
/******** Custom API (Controller) *****/

// Send the command received from the serial monitor to ESP8266
void sendCommandAT(String commandement, unsigned long timeout, bool attendsOK, bool attendPlus, bool imprimer) {
  Serial.println(commandement);
  _dataRecived += showResponseCommandAT(timeout, attendsOK, attendPlus);
  if (imprimer) {
    if (0 == _dataRecived.length()) {
      Serial.println("ESP8266: No Response");
    } else {
      Serial.println(_dataRecived);
      _dataRecived = "";
      Serial.println("-----------------------------------------------------------");
      Serial.println();
      //delay(2000);
    }
  }
}

// Displays the response of the ESP8266 on the serial monitor, with accelerators
String showResponseCommandAT(unsigned long timeout, bool attendsOK, bool attendPlus) {
  if (!attendsOK && !attendPlus) {
    // Something in the header stops ESP8266 data
    while(!Serial.available()) 
    {
      delay(1);
    }
  }
  
  //Displays each character received
  unsigned long fim = millis() + timeout;
  bool bientot = false;
  bool recivedSomething = false;
  String recived = "";
  char beforeLast = ' ', last = ' ';
  do {
    while (!bientot && Serial.available()) {
      recivedSomething = true;
      beforeLast = last;
    last = (char) Serial.read();
      if(last != '"')
      	recived += last;
      if (attendsOK) {
        bientot = ('O' == beforeLast) && ('K' == last);
      } else if (attendPlus) {
        bientot = ('>' == last);
      } else if (recived.length() >= 32) {
        // Partial impressions so as not to overflow the chain
        Serial.print(recived);
        ///////////////////////////////////////////////////////////
    int lampIdx = recived.indexOf("l:")+2;char l=recived.charAt(lampIdx);
    int servoIdx = recived.indexOf("s:")+2;char s=recived.charAt(servoIdx);
    int motorIdx = recived.indexOf("m:")+2;char m=recived.charAt(motorIdx);
        
        digitalWrite(AMPOULE, (l=='1')? HIGH : LOW );
        
        Servo.write((s=='1')? 90 : 0);
        
        if(windowOpened && m=='0'){//Close Window
        	digitalWrite(DESCENTE,HIGH);
  			digitalWrite(MONTE,LOW);
          	windowOpened=false;
        }if(!windowOpened && m=='1'){//Open Window
        	digitalWrite(MONTE,HIGH);
  			digitalWrite(DESCENTE,LOW);
          	windowOpened=true;
        } 
        ///////////////////////////////////////////////////////////
        recived = "";
      } else {
        bientot = ('}' == beforeLast) && ('\r' == last);
      }
    }
  } while (!bientot && (fim >= millis()));
  
  return recived;
}